@@ -35,12 +35,12 @@ Contains simple socket drivers.  Asyncore bugged (haha, pun!) me.
 from __future__ import division
 
 import os
+import sys
 import time
 import errno
 import threading
 import select
 import socket
-import sys
 
 try:
     import ipaddress  # Python >= 3.3 or backported ipaddress
@@ -82,9 +82,7 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
         self.resetDelay()
         if self.networkGroup.get('ssl').value and 'ssl' not in globals():
             drivers.log.error('The Socket driver can not connect to SSL '
-                              'servers for your Python version.  Try the '
-                              'Twisted driver instead, or install a Python'
-                              'version that supports SSL (2.6 and greater).')
+                              'servers for your Python version.')
             self.ssl = False
         else:
             self.ssl = self.networkGroup.get('ssl').value
@@ -107,9 +105,11 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
         return server
 
     def _handleSocketError(self, e):
+        # 'e is None' means the socket was closed.
+        #
         # (11, 'Resource temporarily unavailable') raised if connect
         # hasn't finished yet.  We'll keep track of how many we get.
-        if e.args[0] != 11 or self.eagains > 120:
+        if e is None or e.args[0] != 11 or self.eagains > 120:
             drivers.log.disconnect(self.currentServer, e)
             if self in self._instances:
                 self._instances.remove(self)
@@ -147,6 +147,7 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
 
     @classmethod
     def _select(cls):
+        timeout = conf.supybot.drivers.poll()
         try:
             if not cls._selecting.acquire(blocking=False):
                 # there's already a thread running this code, abort.
@@ -164,7 +165,7 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
             if not cls._instances:
                 return
             rlist, wlist, xlist = select.select([x.conn for x in cls._instances],
-                    [], [], conf.supybot.drivers.poll())
+                    [], [], timeout)
             for instance in cls._instances:
                 if instance.conn in rlist:
                     instance._read()
@@ -196,7 +197,13 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
     def _read(self):
         """Called by _select() when we can read data."""
         try:
-            self.inbuffer += self.conn.recv(1024)
+            new_data = self.conn.recv(1024)
+            if not new_data:
+                # Socket was closed
+                self._handleSocketError(None)
+                return
+
+            self.inbuffer += new_data
             self.eagains = 0 # If we successfully recv'ed, we can reset this.
             lines = self.inbuffer.split(b'\n')
             self.inbuffer = lines.pop()
@@ -223,10 +230,11 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
     def connect(self, **kwargs):
         self.reconnect(reset=False, **kwargs)
 
-    def reconnect(self, wait=False, reset=True):
+    def reconnect(self, wait=False, reset=True, server=None):
         self._attempt += 1
         self.nextReconnectTime = None
         if self.connected:
+            self.onDisconnect()
             drivers.log.reconnect(self.irc.network)
             if self in self._instances:
                 self._instances.remove(self)
@@ -242,9 +250,12 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
         else:
             drivers.log.debug('Not resetting %s.', self.irc)
         if wait:
+            if server is not None:
+                # Make this server be the next one to be used.
+                self.servers.insert(0, server)
             self.scheduleReconnect()
             return
-        self.server = self._getNextServer()
+        self.currentServer = server or self._getNextServer()
         network_config = getattr(conf.supybot.networks, self.irc.network)
         socks_proxy = network_config.socksproxy()
         try:
@@ -254,20 +265,20 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
             log.error('Cannot use socks proxy (SocksiPy not installed), '
                     'using direct connection instead.')
             socks_proxy = ''
-        if socks_proxy:
-            address = self.server[0]
         else:
             try:
-                address = utils.net.getAddressFromHostname(self.server[0],
-                        attempt=self._attempt)
+                hostname = utils.net.getAddressFromHostname(
+                    self.currentServer.hostname,
+                    attempt=self._attempt)
             except (socket.gaierror, socket.error) as e:
                 drivers.log.connectError(self.currentServer, e)
                 self.scheduleReconnect()
                 return
-        port = self.server[1]
         drivers.log.connect(self.currentServer)
         try:
-            self.conn = utils.net.getSocket(address, port=port,
+            self.conn = utils.net.getSocket(
+                    self.currentServer.hostname,
+                    port=self.currentServer.port,
                     socks_proxy=socks_proxy,
                     vhost=conf.supybot.protocols.irc.vhost(),
                     vhostv6=conf.supybot.protocols.irc.vhostv6(),
@@ -282,17 +293,20 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
         try:
             # Connect before SSL, otherwise SSL is disabled if we use SOCKS.
             # See http://stackoverflow.com/q/16136916/539465
-            self.conn.connect((address, port))
-            if network_config.ssl():
+            self.conn.connect(
+                (self.currentServer.hostname, self.currentServer.port))
+            if network_config.ssl() or \
+                    self.currentServer.force_tls_verification:
                 self.starttls()
 
             # Suppress this warning for loopback IPs.
-            targetip = address
+            targetip = hostname
             if sys.version_info[0] < 3:
                 # Backported Python 2 ipaddress demands unicode instead of str
                 targetip = targetip.decode('utf-8')
             elif (not network_config.requireStarttls()) and \
                     (not network_config.ssl()) and \
+                    (not self.currentServer.force_tls_verification) and \
                     (ipaddress is None or not ipaddress.ip_address(targetip).is_loopback):
                 drivers.log.warning(('Connection to network %s '
                     'does not use SSL/TLS, which makes it vulnerable to '
@@ -301,10 +315,8 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
                     '<http://docs.limnoria.net/en/latest/use/faq.html#how-to-make-a-connection-secure>')
                     % self.irc.network)
 
-            def setTimeout():
-                self.conn.settimeout(conf.supybot.drivers.poll())
-            conf.supybot.drivers.poll.addCallback(setTimeout)
-            setTimeout()
+            conf.supybot.drivers.poll.addCallback(self.setTimeout)
+            self.setTimeout()
             self.connected = True
             self.resetDelay()
         except socket.error as e:
@@ -321,6 +333,12 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
             return
         self._instances.append(self)
 
+    def setTimeout(self):
+        try:
+            self.conn.settimeout(conf.supybot.drivers.poll())
+        except Exception:
+            drivers.log.exception('Could not set socket timeout:')
+
     def _checkAndWriteOrReconnect(self):
         self.writeCheckTime = None
         drivers.log.debug('Checking whether we are connected.')
@@ -347,12 +365,15 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
     def die(self):
         if self in self._instances:
             self._instances.remove(self)
+        conf.supybot.drivers.poll.removeCallback(self.setTimeout)
         self.zombie = True
         if self.nextReconnectTime is not None:
             self.nextReconnectTime = None
         if self.writeCheckTime is not None:
             self.writeCheckTime = None
         drivers.log.die(self.irc)
+        drivers.IrcDriver.die(self)
+        drivers.ServersMixin.die(self)
 
     def _reallyDie(self):
         if self.conn is not None:
@@ -363,6 +384,16 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
     def name(self):
         return '%s(%s)' % (self.__class__.__name__, self.irc)
 
+    def anyCertValidationEnabled(self):
+        """Returns whether any kind of certificate validation is enabled, other
+        than Server.force_tls_verification."""
+        network_config = getattr(conf.supybot.networks, self.irc.network)
+        return any([
+            conf.supybot.protocols.ssl.verifyCertificates(),
+            network_config.ssl.serverFingerprints(),
+            network_config.ssl.authorityCertificate(),
+        ])
+
     def starttls(self):
         assert 'ssl' in globals()
         network_config = getattr(conf.supybot.networks, self.irc.network)
@@ -375,39 +406,34 @@ class SocketDriver(drivers.IrcDriver, drivers.ServersMixin):
             drivers.log.warning('Could not find cert file %s.' %
                     certfile)
             certfile = None
-        verifyCertificates = conf.supybot.protocols.ssl.verifyCertificates()
-        if not verifyCertificates:
-            drivers.log.warning('Not checking SSL certificates, connections '
-                    'are vulnerable to man-in-the-middle attacks. Set '
-                    'supybot.protocols.ssl.verifyCertificates to "true" '
-                    'to enable validity checks.')
+        if self.currentServer.force_tls_verification \
+                and not self.anyCertValidationEnabled():
+            verifyCertificates = True
+        else:
+            verifyCertificates = conf.supybot.protocols.ssl.verifyCertificates()
+            if not self.currentServer.force_tls_verification \
+                    and not self.anyCertValidationEnabled():
+                drivers.log.warning('Not checking SSL certificates, connections '
+                        'are vulnerable to man-in-the-middle attacks. Set '
+                        'supybot.protocols.ssl.verifyCertificates to "true" '
+                        'to enable validity checks.')
         try:
             self.conn = utils.net.ssl_wrap_socket(self.conn,
-                    logger=drivers.log, hostname=self.server[0],
+                    logger=drivers.log,
+                    hostname=self.currentServer.hostname,
                     certfile=certfile,
                     verify=verifyCertificates,
                     trusted_fingerprints=network_config.ssl.serverFingerprints(),
                     ca_file=network_config.ssl.authorityCertificate(),
                     )
-        except getattr(ssl, 'CertificateError', None) as e:
-            # Default to None for old Python version, which do not have
-            # CertificateError
+        except ssl.CertificateError as e:
             drivers.log.error(('Certificate validation failed when '
                 'connecting to %s: %s\n'
                 'This means either someone is doing a man-in-the-middle '
                 'attack on your connection, or the server\'s certificate is '
                 'not in your trusted fingerprints list.')
                 % (self.irc.network, e.args[0]))
-            raise ssl.SSLError('Aborting because of failed certificate '
-                    'verification.')
-        except ssl.SSLError as e:
-            drivers.log.error(('Certificate validation failed when '
-                'connecting to %s: %s\n'
-                'This means either someone is doing a man-in-the-middle '
-                'attack on your connection, or the server\'s '
-                'certificate is not trusted.')
-                % (self.irc.network, e.args[1]))
-            raise ssl.SSLError('Aborting because of failed certificate '
+            raise ssl.CertificateError('Aborting because of failed certificate '
                     'verification.')
 
 
